import codepipeline = require('@aws-cdk/aws-codepipeline-api');
import cdk = require('@aws-cdk/cdk');
import { IFunction } from './function-base';
/**
 * Common properties for creating a {@link PipelineInvokeAction} -
 * either directly, through its constructor,
 * or through {@link IFunction#toCodePipelineInvokeAction}.
 */
export interface CommonPipelineInvokeActionProps extends codepipeline.CommonActionProps {
    /**
     * The optional input Artifacts of the Action.
     * A Lambda Action can have up to 5 inputs.
     * The inputs will appear in the event passed to the Lambda,
     * under the `'CodePipeline.job'.data.inputArtifacts` path.
     *
     * @default the Action will not have any inputs
     * @see https://docs.aws.amazon.com/codepipeline/latest/userguide/actions-invoke-lambda-function.html#actions-invoke-lambda-function-json-event-example
     */
    readonly inputArtifacts?: codepipeline.Artifact[];
    /**
     * The optional names of the output Artifacts of the Action.
     * A Lambda Action can have up to 5 outputs.
     * The outputs will appear in the event passed to the Lambda,
     * under the `'CodePipeline.job'.data.outputArtifacts` path.
     * It is the responsibility of the Lambda to upload ZIP files with the Artifact contents to the provided locations.
     *
     * @default the Action will not have any outputs
     */
    readonly outputArtifactNames?: string[];
    /**
     * String to be used in the event data parameter passed to the Lambda
     * function
     *
     * See an example JSON event in the CodePipeline documentation.
     *
     * https://docs.aws.amazon.com/codepipeline/latest/userguide/actions-invoke-lambda-function.html#actions-invoke-lambda-function-json-event-example
     */
    readonly userParameters?: any;
    /**
     * Adds the "codepipeline:PutJobSuccessResult" and
     * "codepipeline:PutJobFailureResult" for '*' resource to the Lambda
     * execution role policy.
     *
     * NOTE: the reason we can't add the specific pipeline ARN as a resource is
     * to avoid a cyclic dependency between the pipeline and the Lambda function
     * (the pipeline references) the Lambda and the Lambda needs permissions on
     * the pipeline.
     *
     * @see
     * https://docs.aws.amazon.com/codepipeline/latest/userguide/actions-invoke-lambda-function.html#actions-invoke-lambda-function-create-function
     *
     * @default true
     */
    readonly addPutJobResultPolicy?: boolean;
}
/**
 * Construction properties of the {@link PipelineInvokeAction Lambda invoke CodePipeline Action}.
 */
export interface PipelineInvokeActionProps extends CommonPipelineInvokeActionProps {
    /**
     * The lambda function to invoke.
     */
    readonly lambda: IFunction;
}
/**
 * CodePipeline invoke Action that is provided by an AWS Lambda function.
 *
 * @see https://docs.aws.amazon.com/codepipeline/latest/userguide/actions-invoke-lambda-function.html
 */
export declare class PipelineInvokeAction extends codepipeline.Action {
    private readonly props;
    constructor(props: PipelineInvokeActionProps);
    outputArtifacts(): codepipeline.Artifact[];
    outputArtifact(artifactName: string): codepipeline.Artifact;
    protected bind(stage: codepipeline.IStage, _scope: cdk.Construct): void;
}
